@@ -1,8 +1,8 @@
-from __future__ import print_function, division, absolute_import
+from __future__ import absolute_import, division, print_function
 
 import os
-import sys
 import subprocess
+import sys
 
 import pytest
 
